function [BCAci]=BCa(Bsamp,BB,BBB,CIlevel);
% [BCaci]=BCa(Bsamp,BB,BBB,CIlevel);
% Input:
%       Bsamp: contains vector with sample solution
%       BB: contains bootstrap parameters (one column per bootstrap)
%       BBB: contains expanded data parameters (one column per expanded data set)
%       CIlevel: level of confidence interval, e.g., .95, or .90
% Output:
%       BCaci: matrix (nparx2), with npar equals number of parameters, 
%       column 1: lower bound of CI;     column 2: upper bound of CI;
%   
% uses norminv.m and normcdf.m from statistics toolbox

[m,n_boot]=size(BB);
mm=size(Bsamp,1);
if m~=mm,disp(' bootstraps do not have same size as sample');return;end;
[mm,n]=size(BBB);
if m~=mm,disp(' parameter matrices do not have same numbers of rows');return;end;

dp=[(1-CIlevel)/2 1-(1-CIlevel)/2]';        %desired percentiles for BCa method

%Compute z0
za0=norminv(dp',0,1);    
za=ones(mm,1)*za0;


% compute bias parameters
    % count number of cases where bootstrap sample value is smaller 
    % than original sample value; if this is 0, this number is set to 1; 
    % Rationale: in fact original sample should also be in the total
    % distribution of all bootstraps, so we simply add it here
pp=((sum(BB'<(ones(n_boot,1)*(Bsamp(:)')))) +((sum(BB'<(ones(n_boot,1)*(Bsamp(:)'))))==0));
p=(pp./n_boot)';
    % A correction is needed for extreme cases, i.e. where orig. sample value
    % is smaller than or larger than all bootstrap values, to avoid zero width intervals
    % In this case the proportion is obtained by division by n_boot+1
    % which is based on the idea that we consider the orig. sample as one of the bootstraps.  
correction=(((sum(BB'>(ones(n_boot,1)*(Bsamp(:)'))))==0) | ((sum(BB'>(ones(n_boot,1)*(Bsamp(:)'))))==n_boot));
p(correction)=(pp(correction)./(n_boot+1))';
w=norminv(p,0,1);
wm=w*ones(1,2);

% compute acceleration parameters
Av=Bsamp(:)*ones(1,n);
Avns=BBB;
Uv=(Avns-Av)./(n+1);  %Uv (mm,n) 
a= ((sum((Uv.^3)')')./((sum((Uv.^2)')').^1.5))/6;    %a (mm,1): acceleration constant for each loading

% determine BCa upper and lower bounds

am=a*ones(1,2);
ad=(wm+ ((wm+za)./(1-(am.*(wm+za))) ));
atot=(normcdf(ad,0,1))';  %bias corrected and accelerated alfa's; 

[tmp1,tmp2]=size(atot);
for itmp1=1:tmp1
    for itmp2=1:tmp2,
        if isnan(atot(itmp1,itmp2)),
            atot(itmp1,itmp2)=Bsamp(itmp1);
        end;
    end;
end;
[out,pr]=percentiledp(BB',atot); % out(mmx1): desired percentiles for all j columns of BB 

%   correct all extreme cases, by considering original sample as one of the bootstraps
if sum(correction)>0
    outcorr=percentiledp([BB(correction,:) Bsamp(correction)]',atot(:,correction)); 
    out(:,correction)=outcorr;
end;
BCAci=out';

