%function [R,THRES,R0_IC]=polychomap(X,Xmin,Xmax,info,test,disp);
%
% PARAMETERS IN
%
%        X       MATRIZ BINARIA  (N x m)
%        Xmin    VECTOR CON EL VALOR MINIMO DE LAS VARIABLES  (m x 1)
%        Xmax    VECTOR CON EL VALOR MAXIMO DE LAS VARIABLES  (m x 1)
%
%        info    0: noninformative prior, uniform (-1,1) (deafault)
%                1: truncated normal tight prior (sx=0.33)
%                2: truncated normal moderate prior (sx=1)
%                3: truncated normal weak prior (sx=3)
%
%        test    Number of bootstrap samples to be used
%                test: 199  --> 199 bootstrap samples are used to test
%                               null hypothesis and point estimate
%                               confident intervals (90%)
%                test: [199 199] --> 199 bootstrap samples are used to test
%                               null hypothesis, and double bootstrap
%                               is computed to point estimate confident 
%                               intervals (90%); the second value is the
%                               number of sub-boostrap samples
%
%        disp    1: shows progress bars
%                0: hiddes progress bars
%
% PARAMETERS OUT
%
%        R       MATRIZ DE CORRELACIONES TETRACORICAS/POLICORICAS
%        THRES   THRESHOLDS DE LAS VARIABLES
%        RO_IC   Matrix with pair of variables index, point estimates,
%                90% conficent interval of H0, significant coefficients 
%                and 90% confident interval of significant point estimates
%                Example:
%
%    VARIABL  Point E   LI --- H0 --- LS   SIG   LI --- H0 --- LS
%    ======== =======   ================= ====   =================
%    1    2    0.4119   -0.0931    0.0517    1    0.2897    0.4671
%    1    3    0.0119   -0.1552    0.0931    0         0         0
%    1    4    0.0734   -0.0724    0.0931    0         0         0
%    2    3   -0.0307   -0.0724    0.1138    0         0         0
%    2    4   -0.0128   -0.1345    0.0724    0         0         0
%    3    4   -0.4152   -0.1138    0.0517    1   -0.5063   -0.3083
%  
%    =============================================================
%
%        FUNCIONES:
%               tetra.m
%               TableConti.m
%               ppnd.m
%               normalbi.m
%               ordnorb.m
%               ordnormalbi.m
%               bivnor.m
%               fnor.m
%               correc
%               polymapi
%               creano2
%               Time2String
%
% LAST UPDATED : 26/03/2014
%                Urbano Lorenzo-Seva
%
function [R,THRES,R0_IC]=polychomap(X,Xmin,Xmax,info,test,disp);

if nargin<6, disp=0; end;
if nargin<5, test=0;disp=0;end;
if nargin<4, info=0;test=0;disp=0;end;
if nargin<3, 
     fprintf ( 1, 'Polychomap error\n' );
     fprintf ( 1, '' );
     help polychomap
     fprintf ( 1, '' );
     return
end;

[tmp1, tmp2]=size(test);
if max([tmp1; tmp2])>1,
   samp1=test(1);
   samp2=test(2);
else,
   samp1=test;
   samp2=0;
end;
    
[N,m] = size(X);
conv = 1E-8;

Ki = (Xmax - Xmin + 1);
Kmax= max(Ki);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% determines the correction factor for setting the mean
% of the prior distribution

cfac=correc(Ki);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

Rpear=corrcoef(X);

R=eye(m);
THRES=zeros(m,max(Ki')-1);
total = m*(m-1)/2;
R0_IC = zeros(total,8);
percent = 0.90;
interval=percent;

%TO PRECOMPUTE THREHOLDS !!!!!!!!!!!!!!
TH = zeros(m,Kmax+1);
TH05 = zeros(m,Kmax+1);
FR = zeros(m,Kmax);
FR05 = FR + 0.5*Kmax;
for i=1:m,
    Xmi = min(X(:,i));
    for k=1:N,
     jth=1;
     for j=1:Kmax,
        if X(k,i) == (Xmi-1+j), jth = j; end;
     end;
     FR(i,jth)= FR(i,jth) + 1;
    end;
    FR05(i,:)=FR05(i,:) + FR(i,:);
    FR(i,:)=FR(i,:)/N;
    FR05(i,:)=FR05(i,:)/(N + 0.5 * Kmax *Kmax);
    TH(i,1)=-5;
    TH05(i,1)=-5;
    for k=1:Kmax,
       tmp1= sum(FR(i,1:k)');
       TH(i,k+1)=ppnd(tmp1);
       tmp1= sum(FR05(i,1:k)');

       TH05(i,k+1)=ppnd(tmp1);
    end;
    TH(i,Kmax+1)=5;
    TH05(i,Kmax+1)=5;
end;

%bar
%if disp == 1, bar1 = waitbar(0,['Please wait for ' num2str(total) ' polychoric/tetrachoric correlations to be computed...']); end
if disp == 1,
   tstart = tic;
   tmp3 = 0; bar1 = waitbar(tmp3,'Dispersion matrix');
    
end;    
h=0;
ch=0;

for i=1:(m-1),
   for j=i+1:m,

        h=h+1;
        ch=ch+1;
 %       if (disp > 0), waitbar(ch/(total + (total*samp1))); end
           
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       %%%%%%%%%%%%%% %Point estimate
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        Xi = [X(:,i),X(:,j)];
       
        TC = tableconti(Xi,[Xmin(i) Xmin(j)],[Xmax(i) Xmax(j)]);
        if min(min(TC)') == 0,
            TC = TC+0.5;
            tha = TH05(i,:);
            thb = TH05(j,:);
        else,
            tha = TH(i,:);
            thb = TH(j,:);
        end;
       
        if ((Ki(i) == 2) & (Ki(j) == 2)),
            r = tetra(TC(2,1),TC(1,1),TC(2,2),TC(1,2));
            rp=Rpear(i,j);
            r = r*sign(rp);
        else,
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        % sets the mean of the prior distribution (initial estimate)
        % by using the correction factor
        % and calls the bivariate subfunction
            inic=Rpear(i,j)/cfac;
            r = polimapi(TC,tha,thb,[Ki(i) Ki(j)],info,inic,60);  
            if info==0,
                inic=r;
                r = polimapi(TC,tha,thb,[Ki(i) Ki(j)],info,inic,0);  
            end;
        %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        end;
       R(i,j) = r;
       R(j,i) = r;
       
       if (disp >0),
            compT = toc(tstart);
            compT = compT*(total-ch)/ch;
            s_tmp=['Dispersion matrix (Remaining time = ' Time2String( compT ) ')'];
            waitbar(ch/(total + (total*samp1)),bar1,s_tmp);
       end;
       
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
       %%%%%%%%%%%%%% Test Null hypotesis
       %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
        if samp1>0,
            R0_IC(h,1)=i;
            R0_IC(h,2)=j;
            R0_IC(h,3)=r;
            
            rtmp=zeros(samp1,1);
            for hh=1:samp1,
                
                 ch=ch+1;
                 if (disp > 0) waitbar(ch/(total + (total*samp1))); end
                
                [dummy J]=sort(rand(N,2));	%J is a permutation matrix
                 Xii=zeros(N,2);for ii=1:2, Xii(:,ii)=Xi(J(:,ii),ii);end;
                TC = tableconti(Xii,[Xmin(i) Xmin(j)],[Xmax(i) Xmax(j)]);
                if min(min(TC)') == 0,
                    TC = TC+0.5;
                    thaii = TH05(i,:);
                    thbii = TH05(j,:);
                else,
                    thaii = TH(i,:);
                    thbii = TH(j,:);
                end;
                if ((Ki(i) == 2) & (Ki(j) == 2)),
                    r = tetra(TC(2,1),TC(1,1),TC(2,2),TC(1,2));
                    rp=corrcoef(Xii);
                    r = r*sign(rp(2,1));
                else,
                    inic=0.0;  %R=0 in population
                    r = polimapi(TC,thaii,thbii,[Ki(i) Ki(j)],3,inic,30);
                end;
       
                rtmp(hh,1) = r; 
                
            end;
            rtmp = sort(rtmp);     % sorting the eigenvalues for each position.
            J= round((1-percent)/2*samp1);if (J<1),J=1;end;
            R0_IC(h,4) = rtmp(J,1);
            J= round((percent+(1-percent)/2)*samp1);if (J>samp1),J=samp1;end;
            R0_IC(h,5) = rtmp(J,1);
            if (R0_IC(h,3) > R0_IC(h,4)) && (R0_IC(h,3) < R0_IC(h,5)),
                R0_IC(h,6)=0;
            else,
                R0_IC(h,6)=1;
            end;
        end;

       if (R0_IC(h,6) == 1) && (samp1>1), 
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           %%%%%%%%%%%%%% Test Confidence interval: double bootstrap
           %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
           if samp1>1,
               
               if disp == 1, bar2 = waitbar(0,['Please wait for    ' num2str((samp1+samp1*samp2)) '    double resampling computations ... done (0000000000)']); end
               chh=0;
               ri=zeros(samp1,1);
               rii=zeros(samp2,samp1);
               for is=1:samp1,

                    chh=chh+1;
                    if (disp > 0), 
                        if (samp2>0),
                            waitbar(chh/(samp1*samp2),bar2,['Please wait for ' num2str((samp1+samp1*samp2)) ' double resampling computations ... done (' num2str(chh) ')']); 
                        else,
                            waitbar(chh/(samp1),bar2,['Please wait for ' num2str((samp1+samp1*samp2)) ' resampling computations ... done (' num2str(chh) ')']); 
                        end;
                    end;

                    J = round(rand(N,1)*(N-1)+1);
                    Xii = X(J,[i j]);
                     TCi = tableconti(Xii,[Xmin(i) Xmin(j)],[Xmax(i) Xmax(j)]);
                     if min(min(TCi)') == 0,
                        TCi = TCi+0.5;
                     end;
                     tmp = sum(TCi)/sum(sum(TCi)');
                     thaii=zeros(Kmax+1,1);
                     thaii(1)=-5;
                     for k=1:Kmax,
                        tmp1= sum(tmp(1:k)');
                        thaii(k+1)=ppnd(tmp1);
                     end;
                     thaii(Kmax+1)=5;

                     tmp = sum(TCi')/sum(sum(TCi)');
                     thbii  = zeros(Kmax+1,1);
                     thbii(1)=-5;
                     for k=1:Kmax-1,
                        tmp1= sum(tmp(1:k)');
                        thbii(Kmax-k+1)= -1*ppnd(tmp1);
                     end;
                     thbii(Kmax+1)=5;
                     
                    %%%Polychoric MAP
                    if ((Ki(i) == 2) & (Ki(j) == 2)),
                        ri(is,1) = tetra(TCi(2,1),TCi(1,1),TCi(2,2),TCi(1,2));
                        rp=corrcoef(Xi);
                        ri(is,1) = ri(is,1)*sign(rp(2,1));
                    else,
                        inic=corrcoef(Xii);
                        inic=inic(2,1)/cfac;
                        ri(is,1) = polimapi(TCi,thaii,thbii,[Ki(i) Ki(j)],3,inic,30);
                    end;

                    if samp2 > 0,
                        for ii=1:samp2,
                            chh=chh+1;
                            if (disp > 0), waitbar(chh/(samp1*samp2),bar2,['Please wait for ' num2str((samp1+samp1*samp2)) ' resampling computations ... done (' num2str(chh) ')']); end
                             J = round(rand(N,1)*(N-1)+1);
                             Xiii = Xii(J,:);

                             TCi = tableconti(Xiii,[Xmin(i) Xmin(j)],[Xmax(i) Xmax(j)]);
                             if min(min(TCi)') == 0,
                                TCi = TCi+0.5;
                             end;
                             tmp = sum(TCi)/sum(sum(TCi)');
                             thaii=zeros(Kmax+1,1);
                             thaii(1)=-5;
                             for k=1:Kmax,
                                tmp1= sum(tmp(1:k)');
                                thaii(k+1)=ppnd(tmp1);
                             end;
                             thaii(Kmax+1)=5;

                             tmp = sum(TCi')/sum(sum(TCi)');
                             thbii  = zeros(Kmax+1,1);
                             thbii(1)=-5;
                             for k=1:Kmax-1,
                                tmp1= sum(tmp(1:k)');
                                thbii(Kmax-k+1)= -1*ppnd(tmp1);
                             end;
                             thbii(Kmax+1)=5;

                            if ((Ki(i) == 2) & (Ki(j) == 2)),
                                rii(ii,is) = tetra(TCi(2,1),TCi(1,1),TCi(2,2),TCi(1,2));
                                rp=corrcoef(Xii);
                                rii(ii,is) = rii(ii,si)*sign(rp(2,1));
                            else,
                                inic=corrcoef(Xiii);
                                inic=inic(2,1)/cfac;
                                rii(ii,is) = polimapi(TCi,thaii,thbii,[Ki(i) Ki(j)],3,inic,30);
                            end;
                        end;
                        rii(:,is)=sort(rii(:,is));
                    end;
                    
                end;
                ri=sort(ri);
                if disp == 1,close(bar2);end;
                
                %coverage for doble bootstrap
                if samp2>0,
                    IC=zeros(1,2);
                    p=interval-0.01;
                    intervf=0;
                    while (p<1),
                        cove=0;
                        for ii=1:samp1,
                            J= round((1-p)/2*samp2);if (J<1),J=1;end;
                            IC(1,1) = rii(J,ii);
                            J= round((p+(1-p)/2)*samp2);if (J>samp2),J=samp2;end;
                            IC(1,2) = rii(J,ii);
                            if (R(i,j)>IC(1,1) & R(i,j)<IC(1,2)),
                                cove=cove+1;
                            end;
                        end;
                        if (cove/samp1 > interval),
                            intervf=p;
                            p=1;
                        else,
                            p=p+0.001;
                        end;
                    end;
                else,
                    intervf=interval;
                end;

                J= round((1-intervf)/2*samp1);if (J<1),J=1;end;
                R0_IC(h,7)=ri(J,1);
                J= round((intervf+(1-intervf)/2)*samp1);if (J>samp1),J=samp1;end;
                R0_IC(h,8)=ri(J,1);

           end;
       end;
       
   end; 
    
    for k=2:Kmax,
       THRES(i,k-1) = tha(k);
    end;
    
end;

for k=2:Kmax,
    THRES(m,k-1) = thb(k);
end;

if disp == 1,close(bar1);end;

return;
