%        DOUBLE PRECISION FUNCTION PPND(P,IER)
%
%       ALGORITHM AS 111, APPL.STATIST., VOL.26, 118-121, 1977.
%
%       PRODUCES NORMAL DEVIATE CORRESPONDING TO LOWER TAIL AREA = P.
%
%	See also AS 241 which contains alternative routines accurate to
%	about 7 and 16 decimal digits.
%
function [PPND,IER] = ppnd(P);

SPLIT=0.42;
A0 =  2.50662823884D0;
A1 =-18.61500062529;
A2 = 41.39119773534;
A3 =-25.44106049637;
B1 = -8.47351093090;
B2 = 23.08336743743;
B3 =-21.06224101826;
B4 =  3.13082909833;
C0 = -2.78718931138;
C1 = -2.29796479134;
C2 =  4.85014127135;
C3 =  2.32121276858;
D1 = 3.54388924762;
D2 = 1.63706781897;
ZERO = 0.0;
ONE  = 1.0;
HALF = 0.5;

        IER = 0;
        Q = P-HALF;
        if (abs(Q) <=SPLIT),
%
%           0.08 < P < 0.92
%
            R = Q*Q;
            PPND = Q*(((A3*R + A2)*R + A1)*R + A0)/((((B4*R + B3)*R + B2)*R + B1)*R + ONE);
            return;
        end;

%
%       P < 0.08 OR P > 0.92, SET R = MIN(P,1-P)
%
        R = P;
        if (Q > ZERO), R = ONE-P; end;
        if (R>ZERO),
            R = sqrt(-log(R));
            PPND = (((C3*R + C2)*R + C1)*R + C0)/((D2*R + D1)*R + ONE);
            if (Q < ZERO), PPND = -PPND; end;
            return;
        end;

        IER = 1;
        PPND = ZERO;
        return;


