%
%        ALGORITHM  AS 116 APPL. STATIST. (1977) VOL.26, NO.3
%
%        TO COMPUTE THE TETRACHORIC CORRELATION (R) 
%        FROM THE FREQUENCIES OF A 2*2 TABLE
%        (A, B, C AND D)
%
%               V2
%                0    1
%        V1 0    A    B
%           1    C    D
%
function [R,THRESHOLDS]=tetra(A,B,C,D);

ZERO   =0.0;
ONE    =1.0;
TWO    =2.0;
THREE  =3.0;
FOUR   =4.0;
SIX    =6.0;
HALF   =0.5;
TWOPI  =6.28318531;
SQT2PI =2.50662827;
RLIMIT =0.9999;
RCUT   =0.95;
UPLIM  =5.0;
CONST  =1E-36;
CHALF  =1E-18;
CONV   =1E-8;
CITER  =1E-6;
NITER  =10;

%
%        INITIALIZATION
%

      R = ZERO;
      SDZERO = ZERO;
      SDR = ZERO;
      ITYPE = 0;
      IFAULT = 0;
%
%       CHECK IF ANY CELL FREQUENCY IS NEGATIVE
%
      if ((A < ZERO) | (B < ZERO) | (C < ZERO) | (D < ZERO)),
         ITYPE = 2;
         disp('Una frequencia es negativa!');
         return;
      end;

%
%       CHECK IF ANY FREQUENCY IS ZERO AND SET KDELTA
%
      KDELTA = ONE;
      DELTA  = ZERO;
      if ((A == ZERO) | (D == ZERO)), KDELTA  = 2; end;
      if ((B == ZERO) | (C == ZERO)), KDELTA  = KDELTA + 2; end;

%
%        KDELTA=4 MEANS TABLE HAS ZERO ROW OR COLUMN, RUN IS TERMINATED
%
      if (KDELTA == FOUR),
         ITYPE = 2;
         disp('La taula te una fila o una columna plena de zeros!');
         return;
      end;
%
%        DELTA IS 0.0, 0.5 OR -0.5 ACCORDING TO WHICH CELL IS ZERO
%
    if (KDELTA == TWO),
       DELTA  = HALF;
       if (A == ZERO) &  (D == ZERO), R = -ONE; end;
    end;
    if (KDELTA == THREE),
       DELTA  = -HALF;
       if (B == ZERO) &  (C == ZERO), R =  ONE; end;
    end;
    if (R ~= ZERO), ITYPE = 3; end;

%=====================================================================
%        WHEN R IS 0.0, 1.0 OR -1.0, THE FUNCTION FINISH
%
    if (R ~= ZERO) | (ITYPE > 0), return;  end;

%=====================================================================
%
%        WHEN MARGINALS ARE EQUAL, COSINE EVALUATION IS USED
%
      if ((A == D) & (B == C)),
        R = -COS(TWOPI * PROBAA);
        ITYPE = 2;
        return;
      end;
%=====================================================================
%
%
%        STORE FREQUENCIES IN  AA, BB, CC AND DD
%
      AA = A + DELTA;
      BB = B - DELTA;
      CC = C - DELTA;
      DD = D + DELTA;
      TOT = AA + BB + CC + DD;
%
%        CHECK IF CORRELATION IS NEGATIVE, ZERO, POSITIVE
%        COMPUTE PROBABILITIES OF QUADRANT AND OF MARGINALS
%        PROBAA AND PROBAC CHOSEN SO THAT CORRELATION IS POSITIVE.
%        KSIGN INDICATES WHETHER QUADRANTS HAVE BEEN SWITCHED
%
      TMP = (AA * DD - BB * CC);
      if TMP == 0,
         ITYPE = 4;
         PROBAA = AA / TOT;
         PROBAC = (AA + CC) / TOT;
         KSIGN = ONE;
      end;
      if TMP > 0,
         PROBAA = AA / TOT;
         PROBAC = (AA + CC) / TOT;
         KSIGN = ONE;
      end;
      if TMP < 0,
         PROBAA = BB / TOT;
         PROBAC = (BB + DD) / TOT;
         KSIGN = TWO;
      end;
      PROBAB = (AA + BB) / TOT;
%
%        COMPUTE NORMAL DEVIATES FOR THE MARGINAL FREQUENCIES
%        SINCE NO MARGINAL CAN BE ZERO, IE IS NOT CHECKED
%
      [ZAC,IE] = ppnd(PROBAC);
      [ZAB,IE] = ppnd(PROBAB);
      THRESHOLDS=[ZAB,ZAC];
      SS = exp(-HALF * (ZAC^2 + ZAB^2)) / TWOPI;

%
%        INITIAL ESTIMATE OF CORRELATION IS YULES Y
%
      RR = (sqrt(AA * DD) - sqrt(BB * CC))^2 / abs(AA * DD - BB * CC);
      ITER = 0;

%
%    TETRACHORIC SERIES IS COMPUTED
%

     for ITER=1:NITER,

%         NO IMPLEMENTAT:
%         IF RR EXCEEDS RCUT (0.95), GAUSSIAN QUADRATURE IS USED

%
%         INITIALIZATION
%
          VA = ONE;
          VB = ZAC;
          WA = ONE;
          WB = ZAB;
          TERM = ONE;
          ITERM = 0;
          SUM = PROBAB * PROBAC;
          DERIV = ZERO;
          SR = SS;

          while ((ITERM <2) | (TERM < SIX)),
%
%             RESCALE TERMS TO AVOID OVERFLOWS AND UNDERFLOWS
%
              if (abs(SR) <= CONST),
                 SR = SR / CONST;
                 VA = VA * CHALF;
                 VB = VB * CHALF;
                 WA = WA * CHALF;
                 WB = WB * CHALF;
              end;
%
%             FORM SUM AND DERIVATIVE OF SERIES
%
              DR = SR * VA * WA;
              SR = SR * RR / TERM;
              COF = SR * VA * WA;
%
%             ITERM COUNTS NO. OF CONSECUTIVE TERMS .LT. CONV
%
              ITERM = ITERM + 1;
              if (abs(COF) > CONV), ITERM = 0; end;
              SUM = SUM + COF;
              DERIV = DERIV + DR;
              VAA = VA;
              WAA = WA;
              VA = VB;
              WA = WB;
              VB = ZAC * VA - TERM * VAA;
              WB = ZAB * WA - TERM * WAA;
              TERM = TERM + ONE;
          end;

%
%        CHECK IF ITERATION CONVERGED
%
         if (abs(SUM-PROBAA) <= CITER),
%
%             ITERATION HAS CONVERGED, SET R AND ITYPE
%
              ITYPE = TERM;
              R=RR;
              return;
         end;
%
%        CALCULATE NEXT ESTIMATE OF CORRELATION
%
         DELTA = (SUM - PROBAA) / DERIV;
         RRPREV = RR;
         RR = RR - DELTA;
         if (ITER == 1),   RR = RR + HALF * DELTA; end;
         if (RR > RLIMIT), RR = RLIMIT;            end;
         if (RR < ZERO),   RR = ZERO;              end;

     end;
%
%    SORTIDA SENSE CONVERGENCIA
%
     ITYPE = 2;
     disp('Maxim diteracions superat!');
     R=ZERO;

     return;
